# -*- coding: utf-8 -*-

import json
import time
import MySQLdb
import logging
import logging.config
from artix.filequeue.filequeuemanager import FileQueueManager

logging.config.fileConfig('maint_log.ini')
log = logging.getLogger('su.artix.pricedeleteservice')


def chunks(chunkable, n):
    for i in xrange(0, len(chunkable), n):
        yield chunkable[i:i+n]


def createTransaction(log, commandList, cashCode):
    queueName = 'artix.loaddict.cash'
    try:
        fileQueueManager = FileQueueManager(queueRoot='/linuxcash/cash/data/queues', log=log)
        fileQueueManager.removeUncompletedTransactions(queueName, 'maint')
        transactionId = fileQueueManager.beginTransaction(queueName)
        fileQueueManager.enqueue(queueName, transactionId, {'properties': {'type': 'datagram'}, 'content': commandList})
        endTransProperties = {
            'type': 'control',
            'control': 'end_tran',
            'cashCode': cashCode,
            'statusServer': 'localhost:8080',
            'source': 'maint',
            'transactionId': transactionId,
            'objectsCount': len(commandList)
            }
        fileQueueManager.endTransaction(queueName, transactionId, {'properties': endTransProperties})
    except Exception as e:
        log.error('Ошибка: %s' % str(e))


def calculateData():
    conn = None
    cur = None
    try:
        registryPath = '/linuxcash/cash/data/cash.reg'
        with open(registryPath) as data_file:
            data = json.load(data_file)
            cashCode = str(data['cashCode'])

        data_file = open('/linuxcash/cash/conf/maint.json')
        data = json.load(data_file)
        resultCount = data['prices']['pricesCount'] if 'prices' in data and 'pricesCount' in data['prices'] else 500
        timeout = data['prices']['timeout'] if 'prices' in data and 'timeout' in data['prices'] else 240
        log.info('Начало выборки данных из бд')
        conn = MySQLdb.connect(host='localhost', user='root', passwd='', db='dictionaries')
        cur = conn.cursor()
        cur.execute('set names utf8')
        cur.execute("SELECT code FROM tmc")
        codeList = [item[0] for item in cur.fetchall()]
        log.info('Выбрали все коды из таблицы tmc')
        cur.execute("SELECT barcode FROM barcodes")
        codeList.extend([item[0] for item in cur.fetchall()])
        log.info('Выбрали все штрих-коды из таблицы barcodes')

        cur.execute("SELECT DISTINCT(barcode) FROM dictionaries.prices WHERE effectivedateend < now()")
        result = [{'command': 'deleteOverduePrices', 'barcode': str(item[0])} for item in cur.fetchall()]
        log.info(str(result))
        for data in chunks(result, resultCount):
            log.info('Посылаем %d команд в nes' % len(data))
            createTransaction(log, data, cashCode)
            time.sleep(timeout)

        commandList = []
        for row in chunks(codeList, 50):
            row = filter(None, row)
            query_values = ', '.join("'%s'" % it for it in row)
            query = "SELECT barcode, max(effectivedate), effectivedateend FROM dictionaries.prices \
                    WHERE barcode IN (%s) AND effectivedate < now() AND (effectivedateend > now() OR effectivedateend IS NULL) \
                    GROUP BY barcode, effectivedateend HAVING count(barcode) > 1" % str(query_values)
            cur.execute(query)
            result = [
                {'command': 'deletePriceByDate',
                 'barcode': str(item[0]),
                 'effectivedate': str(item[1]),
                 'effectivedateend': str(item[2]) if item[2] else None}
                for item in cur.fetchall()]

            if result:
                log.info(str(result))
                commandList.extend(result)
                log.info('Сформировали %d команд для nes. Общий размер пакета %d' % (len(result), len(commandList)))

                if len(commandList) >= resultCount:
                    for data in chunks(commandList, resultCount):
                        log.info('Посылаем %d команд в nes' % len(data))
                        createTransaction(log, data, cashCode)
                        time.sleep(timeout)
                    commandList = []
        if commandList:
            createTransaction(log, commandList, cashCode)
    except Exception as e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn and conn.open:
            if cur:
                cur.close()
            conn.close()


def main():
    log.info('Сервис maint_prices начал работу')
    calculateData()
    log.info('Сервис maint_prices закончил работу')


if __name__ == '__main__':
    main()
